package ece476controlmapper;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;

public class ControlMapperGUI extends JFrame implements ActionListener, KeyListener, WindowListener{
	private static final long serialVersionUID = 2851451823991193205L;

	boolean started = false;
	
	SerialConnection connection;
	ControlMapper mapper;

	int height;
	int width;
	
	JPanel mainPane;

	JTextField portNumField;
	JButton startButton;
	JCheckBox showLogCheckBox;
	boolean showLog = true;
	JPanel mappingPane;
	JTextArea status;
	
	JFileChooser fileChooser;
	
	public ControlMapperGUI() {
		super("Hand Gesture Contol Mapper");

		connection = new SerialConnection();
		mapper = new ControlMapper(this);
		connection.addHandler(mapper);
		
		createGUI();
	}

	public void createGUI() {
		height = 300;
		width =  540;
		
		mainPane = new JPanel(new BorderLayout(10, 0));
		mainPane.setPreferredSize(new Dimension(width, height));
		
		// controls ~  port number and start button
		JPanel controlPane = new JPanel(new BorderLayout());
		JLabel portNumLabel = new JLabel("Port: ");
		portNumLabel.setBackground(Color.WHITE);
		portNumField = new JTextField("COM8");
		portNumField.setHorizontalAlignment(JTextField.RIGHT);
		JPanel p = new JPanel(new BorderLayout());
		p.add(portNumLabel, BorderLayout.WEST);
		p.add(portNumField, BorderLayout.CENTER);
		p.setBorder(BorderFactory.createEmptyBorder(10, 0, 10, 0));
		p.setBackground(Color.WHITE);
		controlPane.add(p, BorderLayout.NORTH);
		
		startButton = new JButton("Start");
		startButton.addActionListener(this);
		controlPane.add(startButton, BorderLayout.CENTER);
		
		showLogCheckBox = new JCheckBox("Show Output");
		showLogCheckBox.setBorder(BorderFactory.createEmptyBorder(10, 0, 10, 0));
		showLogCheckBox.setBackground(Color.WHITE);
		showLogCheckBox.setSelected(true);
		showLogCheckBox.addActionListener(this);
		controlPane.add(showLogCheckBox, BorderLayout.SOUTH);
		
		controlPane.setPreferredSize(new Dimension(100, 120));
		controlPane.setBackground(Color.WHITE);
		
		// output area
		status = new JTextArea();
		status.setLineWrap(true);
		status.setRows(5);
		status.setWrapStyleWord(true);
		status.setEditable(false);
		JScrollPane outputPane = new JScrollPane(status);
		outputPane.setBackground(Color.WHITE);
		outputPane.setBorder(BorderFactory.createTitledBorder("Output"));
		
		JPanel commPane = new JPanel(new BorderLayout(10, 0));
		commPane.add(controlPane, BorderLayout.WEST);
		commPane.add(outputPane, BorderLayout.CENTER);
		commPane.setBackground(Color.WHITE);
		mainPane.add(commPane, BorderLayout.PAGE_END);
		
		mappingPane = new JPanel();
		mappingPane.setAlignmentX(LEFT_ALIGNMENT);
		setMappingPaneContent();
		
		mappingPane.setBackground(Color.WHITE);
		mappingPane.setBorder(BorderFactory.createTitledBorder("Key Mappings"));
		mainPane.add(mappingPane, BorderLayout.CENTER);
		
		mainPane.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		mainPane.setBackground(Color.WHITE);
		
		setContentPane(mainPane);
		
		// Create menu bar
		JMenuBar menuBar = new JMenuBar();
		JMenu menu = new JMenu("File");
		JMenuItem load = new JMenuItem("Load Mapping");
		JMenuItem save = new JMenuItem("Save Mapping");
		load.addActionListener(this);
		save.addActionListener(this);
		menu.add(load);
		menu.add(save);
		menuBar.add(menu);
		setJMenuBar(menuBar);
		
		File curdir = new File(".");
		fileChooser = new JFileChooser(curdir.getAbsolutePath());
		fileChooser.addChoosableFileFilter(new MappingFileFilter());
		
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		pack();
		setVisible(true);
	}
	
	private void setMappingPaneContent() {
		mappingPane.removeAll();
		for (GestureControl g : mapper.getGestureList()) {
			mappingPane.add(this.createGestureMapGUIGroup(g));
		}
		mappingPane.setAlignmentX(LEFT_ALIGNMENT);
		mappingPane.validate();
		pack();
	}
	
	private static String[] actTypeStrings = new String[] {"S", "P", "R", "L"};
	private JPanel createGestureMapGUIGroup(GestureControl g) {
		JPanel p = new JPanel(new BorderLayout(2, 0));
		
		JCheckBox cb = new JCheckBox(g.getName());
		cb.setSelected(g.isEnabled());
		cb.setName(g.getId());
		cb.addActionListener(this);
		cb.setBackground(Color.WHITE);
		cb.setPreferredSize(new Dimension(100, 24));
		p.add(cb, BorderLayout.WEST);
		
		JTextField mapField = new JTextField(g.getKeyText());
		mapField.setName(g.getId());
		mapField.addKeyListener(this);
		mapField.setPreferredSize(new Dimension(60, 24));
		p.add(mapField, BorderLayout.CENTER);
		
		JComboBox actList = new JComboBox(actTypeStrings);
		actList.setName(g.getId());
		actList.setSelectedIndex(g.getActionType());
		actList.addActionListener(this);
		actList.setBackground(Color.WHITE);
		p.add(actList, BorderLayout.EAST);

		p.setBackground(Color.WHITE);
		p.setBorder(BorderFactory.createEmptyBorder(5, 8, 5, 8));
		p.setPreferredSize(new Dimension(240, 30));
		return p;
	}
	
	public void print(String message) {
		if(!showLog) return;
		status.setText(status.getText() + message);
	}
	
	public void println(String message) {
		print(message + "\n");
	}
	
	public void showErrorMessage(String message) {
		JOptionPane.showMessageDialog(this, message);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == startButton) {
			if (started) {
				connection.disconnect();
				println("Closed.");
				started = false;
				startButton.setText("Start");
			}
			else {
				try {
					connection.connect(portNumField.getText().trim());
					println(portNumField.getText().trim() + " Opened.");
					started = true;
					startButton.setText("Stop");
				} catch (Exception e1) {
					JOptionPane.showMessageDialog(this, "Exception! " + e1.getMessage());
				} catch (Error e2) {
					JOptionPane.showMessageDialog(this, "Error! " + e2.getMessage());
				}
			}
		}
		else if (e.getSource() == showLogCheckBox) {
			showLog = showLogCheckBox.isSelected();
		}
		else if (e.getSource() instanceof JCheckBox) {
			JCheckBox cb = (JCheckBox)e.getSource();
			String id = cb.getName();
			GestureControl g = mapper.getGesture(id);
			if (g!=null) {
				g.setEnabled(cb.isSelected());
			}
		}
		else if (e.getSource() instanceof JComboBox) {
			JComboBox cb = (JComboBox)e.getSource();
			String id = cb.getName();
			GestureControl g = mapper.getGesture(id);
			if (g!=null) {
				g.setActionType(cb.getSelectedIndex());
			}
		}
		else if (e.getActionCommand().equals("Load Mapping")) {
			int returnVal = fileChooser.showOpenDialog(this);
	        if (returnVal == JFileChooser.APPROVE_OPTION) {
	            File file = fileChooser.getSelectedFile();
	            mapper.load(file);
	            setMappingPaneContent();
	            println("Mapping loaded.");
	        }
		}
		else if (e.getActionCommand().equals("Save Mapping")) {
			int returnVal = fileChooser.showSaveDialog(this);
	        if (returnVal == JFileChooser.APPROVE_OPTION) {
	            File file = fileChooser.getSelectedFile();
	            if (getExtension(file) == null) {
	            	file = new File(file.getAbsolutePath() + ".map");
	            }
	            mapper.write(file);
	            println("Mapping saved.");
	        }
		}
		
	}

	@Override
	public void keyPressed(KeyEvent key) {
	}

	@Override
	public void keyReleased(KeyEvent key) {
		if (started || key.getKeyCode() == KeyEvent.VK_SHIFT) return;
		Component c = this.getFocusOwner();
		if (c != null && c instanceof JTextField) {
			JTextField t = (JTextField)c;
			String id = t.getName();
			GestureControl g = mapper.getGesture(id);
			if (g!=null) {
				if (key.isShiftDown())
					g.addKey(key.getKeyCode());
				else
					g.setKey(key.getKeyCode());
				t.setText(g.getKeyText());
			}
		}
	}

	@Override
	public void keyTyped(KeyEvent arg0) {
	}

	@Override
	public void windowActivated(WindowEvent arg0) {
	}

	@Override
	public void windowClosed(WindowEvent arg0) {
	}

	@Override
	public void windowClosing(WindowEvent arg0) {
	}

	@Override
	public void windowDeactivated(WindowEvent arg0) {
	}

	@Override
	public void windowDeiconified(WindowEvent arg0) {
	}

	@Override
	public void windowIconified(WindowEvent arg0) {
	}

	@Override
	public void windowOpened(WindowEvent arg0) {
	}

	private static class MappingFileFilter extends FileFilter {
		@Override
		public boolean accept(File f) {
			if (f.isDirectory()) {
				return true;
			}
			String extension = getExtension(f);
			if (extension != null) {
				return extension.equals("map");
			}
			return false;
		}
		@Override
		public String getDescription() {
			return "Gesture Control Mapping Files";
		}
	}

	private static String getExtension(File f) {
        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');
        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
        return ext;
    }
}
